@@ -32,6 +32,7 @@ class LogStream {
 	var $port = '';
 	var $user = '';
 	var $database = '';
+	var $preparedStatements = array();
 
 	/**
 	 * append a log line to the log stream
@@ -71,7 +72,7 @@ class LogStream {
 				// if the line number of the added line is 1, we begin a new log block
 				$this->currentBlock = new LogBlock($this, $lineCommandNumber, $line);
 			} else {
-				// otherwise our log file probabbly begins with an incomplete block
+				// otherwise our log file probably begins with an incomplete block
 				// we only raise an error in DEBUG mode as it is very common and is not a blocking problem
 				if(DEBUG) {
 					stderr('we just closed a LogBlock, line number should be 1 and is '.$line->getLineNumber(), true);
@@ -170,6 +171,26 @@ class LogStream {
 		}
 		$this->currentBlock = false;
 	}
+	
+	/**
+	 * add a prepared statement to the list
+	 * 
+	 * @param object $preparedStatement the PreparedStatement object to add
+	 */
+	function addPreparedStatement(& $preparedStatement) {
+		$name = $preparedStatement->getName();
+		$this->preparedStatements[$name] =& $preparedStatement;
+	}
+	
+	/**
+	 * returns a prepared statement by name
+	 * 
+	 * @param string $name name of the prepared statement
+	 * @param string $portalName name of the portal
+	 */
+	function & getPreparedStatement($name, $portalName) {
+		return $this->preparedStatements[$name];
+	}
 }
 
 ?>
\ No newline at end of file
