@@ -2,11 +2,12 @@
 /**
  * GeSHi - Generic Syntax Highlighter
  * 
- * The GeSHi class for Generic Syntax Highlighting. Please refer to the documentation
- * at http://qbnz.com/highlighter/documentation.php for more information about how to
- * use this class.
+ * The GeSHi class for Generic Syntax Highlighting. Please refer to the
+ * documentation at http://qbnz.com/highlighter/documentation.php for more
+ * information about how to use this class.
  *
- * For changes, release notes, TODOs etc, see the relevant files in the docs/ directory
+ * For changes, release notes, TODOs etc, see the relevant files in the docs/
+ * directory.
  *
  *   This file is part of GeSHi.
  *
@@ -28,7 +29,7 @@
  * @author    Nigel McNie <nigel@geshi.org>
  * @copyright Copyright &copy; 2004, 2005, Nigel McNie
  * @license   http://gnu.org/copyleft/gpl.html GNU GPL
- * @version   $Id: geshi.php,v 1.1 2005/11/14 21:19:06 gsmet Exp $
+ * @version   $Id: geshi.php,v 1.3 2006/12/27 19:13:40 gsmet Exp $
  *
  */
 
@@ -40,10 +41,7 @@
 //
 
 /** The version of this GeSHi file */
-define('GESHI_VERSION', '1.0.7.4');
-
-/** For the future (though this may never be realised) */
-define('GESHI_OUTPUT_HTML', 0);
+define('GESHI_VERSION', '1.0.7.16');
 
 /** Set the correct directory separator */
 define('GESHI_DIR_SEPARATOR', ('WIN' != substr(PHP_OS, 0, 3)) ? '/' : '\\');
@@ -126,12 +124,17 @@ define('GESHI_BEFORE', 3);
 /** The key of the regex array defining what bracket group in a
     matched search to put after the replacement */ 
 define('GESHI_AFTER', 4);
+/** The key of the regex array defining a custom keyword to use
+    for this regexp's html tag class */
+define('GESHI_CLASS', 5);
 
 /** Used in language files to mark comments */
 define('GESHI_COMMENTS', 0);
 
 // Error detection - use these to analyse faults
-/** No sourcecode to highlight was specified */
+/** No sourcecode to highlight was specified
+ * @deprecated
+ */
 define('GESHI_ERROR_NO_INPUT', 1);
 /** The language specified does not exist */
 define('GESHI_ERROR_NO_SUCH_LANG', 2);
@@ -196,7 +199,7 @@ class GeSHi
      * @var array
      */
     var $error_messages = array(
-        GESHI_ERROR_NO_INPUT => 'No source code inputted',
+        //GESHI_ERROR_NO_INPUT => 'No source code inputted',
         GESHI_ERROR_NO_SUCH_LANG => 'GeSHi could not find the language {LANGUAGE} (using path {PATH})',
         GESHI_ERROR_FILE_NOT_READABLE => 'The file specified for load_from_file was not readable',
         GESHI_ERROR_INVALID_HEADER_TYPE => 'The header type specified is invalid',
@@ -224,6 +227,7 @@ class GeSHi
      *   a &lt;pre&gt; HTML element.</li>
      *   <li><b>GESHI_HEADER_DIV</b>: Source is outputted in
      *   a &lt;div&gt; HTML element.</li>
+     *   <li><b>GESHI_HEADER_NONE</b>: No header is outputted.</li>
      * </ul>
      * 
      * @var int
@@ -286,6 +290,7 @@ class GeSHi
     /**
      * Whether important blocks should be recognised or not
      * @var boolean
+     * @deprecated
      * @todo REMOVE THIS FUNCTIONALITY!
      */
 	var $enable_important_blocks = false;
@@ -293,6 +298,7 @@ class GeSHi
     /**
      * Styles for important parts of the code
      * @var string
+     * @deprecated
      * @todo As above - rethink the whole idea of important blocks as it is buggy and
      * will be hard to implement in 1.2
      */
@@ -389,12 +395,6 @@ class GeSHi
      */
 	var $encoding = 'ISO-8859-1';
 
-	/**
-     * Unused (planned for future)
-     * @var int
-	 */
-	var $output_format = GESHI_OUTPUT_HTML;
-
     /**#@-*/
 
 	/**
@@ -413,7 +413,7 @@ class GeSHi
 	 */
 	function GeSHi ($source, $language, $path = '')
 	{
-		$this->set_source($source);
+        $this->set_source($source);
         $this->set_language_path($path);
         $this->set_language($language);
 	}
@@ -450,7 +450,7 @@ class GeSHi
 	 */
 	function get_language_name ()
 	{
-		if (GESHI_ERROR_NO_SUCH_LANG == $this->_error) {
+		if (GESHI_ERROR_NO_SUCH_LANG == $this->error) {
 			return $this->language_data['LANG_NAME'] . ' (Unknown Language)';
 		}
 		return $this->language_data['LANG_NAME'];
@@ -464,10 +464,8 @@ class GeSHi
 	 */
 	function set_source ($source)
 	{
-        if ('' == trim($source)) {
-            $this->error = GESHI_ERROR_NO_INPUT;
-        }
 		$this->source = $source;
+        $this->highlight_extra_lines = array();
 	}
 
 	/**
@@ -508,7 +506,8 @@ class GeSHi
 	function set_language_path ($path)
 	{
         if ($path) {
-		  $this->language_path = ('/' == substr($path, strlen($path) - 1, 1)) ? $path : $path . '/';
+			$this->language_path = ('/' == substr($path, strlen($path) - 1, 1)) ? $path : $path . '/';
+			$this->set_language($this->language);	    // otherwise set_language_path has no effect
         }
 	}
 
@@ -533,6 +532,10 @@ class GeSHi
             return;
         }
 		$this->header_type = $type;
+        // Set a default overall style if the header is a <div>
+        if (GESHI_HEADER_DIV == $type && !$this->overall_style) {
+            $this->overall_style = 'font-family: monospace;';
+        }
 	}
 
 	/**
@@ -575,7 +578,7 @@ class GeSHi
 	 */
 	function set_overall_id ($id)
 	{
-		$this->overall_id = $id;
+        $this->overall_id = $id;
 	}
 
 	/**
@@ -1081,11 +1084,12 @@ class GeSHi
 				'asm' => array('ash', 'asm'),
 				'asp' => array('asp'),
 				'bash' => array('sh'),
-				'c' => array('c'),
-				'c_mac' => array('c'),
+				'c' => array('c', 'h'),
+				'c_mac' => array('c', 'h'),
 				'caddcl' => array(),
 				'cadlisp' => array(),
-				'cpp' => array('cpp'),
+                'cdfg' => array('cdfg'),
+				'cpp' => array('cpp', 'h', 'hpp'),
 				'csharp' => array(),
 				'css' => array('css'),
 				'delphi' => array('dpk', 'dpr'),
@@ -1104,6 +1108,7 @@ class GeSHi
 				'php' => array('php', 'php5', 'phtml', 'phps'),
 				'python' => array('py'),
 				'qbasic' => array('bi'),
+                'sas' => array('sas'),
 				'smarty' => array(),
 				'vb' => array('bas'),
 				'vbnet' => array(),
@@ -1305,6 +1310,7 @@ class GeSHi
 	 * Sets whether context-important blocks are highlighted
      * 
      * @todo REMOVE THIS SHIZ FROM GESHI!
+     * @deprecated
 	 */
 	function enable_important_blocks ( $flag )
 	{
@@ -1403,22 +1409,17 @@ class GeSHi
 
 		// Firstly, if there is an error, we won't highlight
 		if ($this->error) {
-			$result = $this->header();
-			if ($this->header_type != GESHI_HEADER_PRE) {
-				$result .= $this->indent(@htmlspecialchars($this->source, ENT_COMPAT, $this->encoding));
-			} else {
-				$result .= @htmlspecialchars($this->source, ENT_COMPAT, $this->encoding);
-			}
-			// Stop Timing
-			$this->set_time($start_time, microtime());
-			return $result . $this->footer();
+            $result = @htmlspecialchars($this->source, ENT_COMPAT, $this->encoding);
+			// Timing is irrelevant
+			$this->set_time($start_time, $start_time);
+			return $this->finalise($result);
 		}
 
-		// Add spaces for regular expression matching and line numbers
-		$code = ' ' . $this->source . ' ';
 		// Replace all newlines to a common form.
-		$code = str_replace("\r\n", "\n", $code);
+		$code = str_replace("\r\n", "\n", $this->source);
 		$code = str_replace("\r", "\n", $code);
+        // Add spaces for regular expression matching and line numbers
+        $code = "\n" . $code . "\n";
 
 		// Initialise various stuff
 		$length           = strlen($code);
@@ -1430,6 +1431,7 @@ class GeSHi
 		$HIGHLIGHTING_ON  = ( !$this->strict_mode ) ? true : '';
 		// Whether to highlight inside a block of code
 		$HIGHLIGHT_INSIDE_STRICT = false;
+		$HARDQUOTE_OPEN = false;
 		$stuff_to_parse   = '';
 		$result           = '';
 
@@ -1538,6 +1540,7 @@ class GeSHi
 					for ($i = 0; $i < $length; $i++) {
 						// Get the next char
 						$char = substr($part, $i, 1);
+				        $hq = isset($this->language_data['HARDQUOTE']) ? $this->language_data['HARDQUOTE'][0] : false;
 						// Is this char the newline and line numbers being used?
 						if (($this->line_numbers != GESHI_NO_LINE_NUMBERS
                             || count($this->highlight_extra_lines) > 0)
@@ -1559,12 +1562,29 @@ class GeSHi
                             // A match of a string delimiter
 							if (($this->lexic_permissions['ESCAPE_CHAR'] && $ESCAPE_CHAR_OPEN) ||
                                 ($this->lexic_permissions['STRINGS'] && !$ESCAPE_CHAR_OPEN)) {
-								$char .= '</span>';
+								$char = htmlspecialchars($char, ENT_COMPAT, $this->encoding) . '</span>';
 							}
+						    $escape_me = false;
+    						if ($HARDQUOTE_OPEN)
+	    					{
+	    						if ($ESCAPE_CHAR_OPEN)
+								$escape_me = true;
+		    					else {
+			    					foreach ($this->language_data['HARDESCAPE'] as $hardesc)
+				    				if (substr($part, $i, strlen($hardesc)) == $hardesc)
+					    			{
+						    			$escape_me = true;
+							    		break;
+							    	}
+							    }
+						    }
 							if (!$ESCAPE_CHAR_OPEN) {
 								$STRING_OPEN = '';
 								$CLOSE_STRING = true;
 							}
+						    if (!$escape_me) {
+						    	$HARDQUOTE_OPEN = false;
+						    }
 							$ESCAPE_CHAR_OPEN = false;
 						} elseif (in_array($char, $this->language_data['QUOTEMARKS']) &&
                             ($STRING_OPEN == '') && $this->lexic_permissions['STRINGS']) {
@@ -1575,21 +1595,55 @@ class GeSHi
 							} else {
 								$attributes = ' class="st0"';
 							}
-							$char = "<span$attributes>" . $char;
+							$char = "<span$attributes>" . htmlspecialchars($char, ENT_COMPAT, $this->encoding);
 
 							$result .= $this->parse_non_string_part( $stuff_to_parse );
 							$stuff_to_parse = '';
-						} elseif (($char == $this->language_data['ESCAPE_CHAR']) && ($STRING_OPEN != '')) {
+    				    } elseif (
+    					$hq &&
+    					substr($part, $i, strlen($hq)) == $hq &&
+    					($STRING_OPEN == '') && $this->lexic_permissions['STRINGS']
+    				    )
+    				    {
+    					// The start of a hard quoted string
+    					$STRING_OPEN = $this->language_data['HARDQUOTE'][1];
+    					if (!$this->use_classes) {
+    					    $attributes = ' style="' . $this->language_data['STYLES']['STRINGS'][0] . '"';
+    					} else {
+    					    $attributes = ' class="st0"';
+    					}
+    					$char = "<span$attributes>" . $hq;
+    					$i += strlen($hq) - 1;
+    					$HARDQUOTE_OPEN = true;
+    					$result .= $this->parse_non_string_part( $stuff_to_parse );
+    					$stuff_to_parse = '';
+    				    } elseif ($char == $this->language_data['ESCAPE_CHAR'] && $STRING_OPEN != '')
+    				    {
                             // An escape character
 							if (!$ESCAPE_CHAR_OPEN) {
+    							$ESCAPE_CHAR_OPEN = !$HARDQUOTE_OPEN;  // true unless $HARDQUOTE_OPEN
+    							if ($HARDQUOTE_OPEN)
+    								foreach ($this->language_data['HARDESCAPE'] as $hard)
+    								{
+    									if (substr($part, $i, strlen($hard)) == $hard)
+    									{
 								$ESCAPE_CHAR_OPEN = true;
-								if ($this->lexic_permissions['ESCAPE_CHAR']) {
+    										break;
+    									}
+    								}
+    							if ($ESCAPE_CHAR_OPEN && $this->lexic_permissions['ESCAPE_CHAR']) {
 									if (!$this->use_classes) {
 										$attributes = ' style="' . $this->language_data['STYLES']['ESCAPE_CHAR'][0] . '"';
 									} else {
 										$attributes = ' class="es0"';
 									}
 									$char = "<span$attributes>" . $char;
+                                    if (substr($code, $i + 1, 1) == "\n") {
+                                        // escaping a newline, what's the point in putting the span around
+                                        // the newline? It only causes hassles when inserting line numbers
+                                        $char .= '</span>';
+                                        $ESCAPE_CHAR_OPEN = false;
+                                    }
 								}
 							} else {
 								$ESCAPE_CHAR_OPEN = false;
@@ -1754,7 +1808,7 @@ class GeSHi
 		$result .= $this->parse_non_string_part($stuff_to_parse);
 
 		// Lop off the very first and last spaces
-		$result = substr($result, 1, strlen($result) - 1);
+		$result = substr($result, 1, -1);
 
 		// Are we still in a string?
 		if ($STRING_OPEN) {
@@ -1785,14 +1839,13 @@ class GeSHi
                     if (false === strpos($line, "\t")) {
                         $lines[$key] = $line;
                         continue;
-                    }//echo 'checking line ' . $key . '<br />';
-
+                    }
+                    
                     $pos = 0;
                     $tab_width = $this->tab_width;
                     $length = strlen($line);
                     $result_line = '';
 
-                    //echo '<pre>line: ' . htmlspecialchars($line) . '</pre>';
                     $IN_TAG = false;
                     for ($i = 0; $i < $length; $i++) {
                         $char = substr($line, $i, 1);
@@ -1811,13 +1864,10 @@ class GeSHi
                             $IN_TAG = true;
                             $result_line .= '<';
                             ++$pos;
-                        } elseif (!$IN_TAG && '&' == $char) {
-                            //echo "matched &amp; in line... ";
-                            $substr = substr($line, $i + 3, 4);
+                        } elseif (!$IN_TAG && '&' == $char) {                            $substr = substr($line, $i + 3, 4);
                             //$substr_5 = substr($line, 5, 1);
                             $posi = strpos($substr, ';');
                             if (false !== $posi) {
-                                //echo "found entity at $posi\n";
                                 $pos += $posi + 3;
                             }
                             $result_line .= '&';
@@ -1830,7 +1880,6 @@ class GeSHi
                             //  3 => '&nbsp; &nbsp;' etc etc
                             // to use instead of building a string every time
                             $strs = array(0 => '&nbsp;', 1 => ' ');
-                            //echo "building (pos=$pos i=$i) (" . ($i - $pos) . ") " . ($tab_width - (($i - $pos) % $tab_width)) . " spaces\n";
                             for ($k = 0; $k < ($tab_width - (($i - $pos) % $tab_width)); $k++) $str .= $strs[$k % 2];
                             $result_line .= $str;
                             //$pos--;
@@ -1839,7 +1888,6 @@ class GeSHi
 
                             if (false === strpos($line, "\t", $i + 1)) {
                                 //$lines[$key] = $result_line;
-                                //echo 'got here';
                                 $result_line .= substr($line, $i + 1);
                                 break;
                             }
@@ -1897,25 +1945,35 @@ class GeSHi
 	 */
 	function add_url_to_keyword ($keyword, $group, $start_or_end)
 	{
-		if (isset($this->language_data['URLS'][$group]) &&
+        if (isset($this->language_data['URLS'][$group]) &&
             $this->language_data['URLS'][$group] != '' &&
             substr($keyword, 0, 5) != '&lt;/') {
 			// There is a base group for this keyword
 			if ($start_or_end == 'BEGIN') {
 				// HTML workaround... not good form (tm) but should work for 1.0.X
-				$keyword = ( substr($keyword, 0, 4) == '&lt;' ) ? substr($keyword, 4) : $keyword;
-				$keyword = ( substr($keyword, -4) == '&gt;' ) ? substr($keyword, 0, strlen($keyword) - 4) : $keyword;
 				if ($keyword != '') {
-					$keyword = ( $this->language_data['CASE_SENSITIVE'][$group] ) ? $keyword : strtolower($keyword);
+                    // Old system: strtolower
+					//$keyword = ( $this->language_data['CASE_SENSITIVE'][$group] ) ? $keyword : strtolower($keyword);
+                    // New system: get keyword from language file to get correct case
+                    foreach ($this->language_data['KEYWORDS'][$group] as $word) {
+                        if (strtolower($word) == strtolower($keyword)) {
+                            break;
+                        }
+                    }
+                    $word = ( substr($word, 0, 4) == '&lt;' ) ? substr($word, 4) : $word;
+                    $word = ( substr($word, -4) == '&gt;' ) ? substr($word, 0, strlen($word) - 4) : $word;
+                    if (!$word) return '';
+                    
 					return '<|UR1|"' .
                         str_replace(
                             array('{FNAME}', '.'),
-                            array(@htmlspecialchars($keyword, ENT_COMPAT, $this->encoding), '<DOT>'),
+                            array(@htmlspecialchars($word, ENT_COMPAT, $this->encoding), '<DOT>'),
                             $this->language_data['URLS'][$group]
                         ) . '">';
 				}
 				return '';
-			} else {
+            // HTML fix. Again, dirty hackage...
+			} elseif (!($this->language == 'html4strict' && '&gt;' == $keyword)) {
 				return '</a>';
 			}
 		}
@@ -1932,8 +1990,8 @@ class GeSHi
 	 */
 	function parse_non_string_part (&$stuff_to_parse)
 	{
-		$stuff_to_parse = ' ' . quotemeta(@htmlspecialchars($stuff_to_parse, ENT_COMPAT, $this->encoding));
-		// These vars will disappear in the future
+		$stuff_to_parse = ' ' . @htmlspecialchars($stuff_to_parse, ENT_COMPAT, $this->encoding);
+        $stuff_to_parse_pregquote = preg_quote($stuff_to_parse, '/');
 		$func = '$this->change_case';
 		$func2 = '$this->add_url_to_keyword';
 
@@ -1944,14 +2002,14 @@ class GeSHi
 			if ($this->lexic_permissions['REGEXPS'][$key]) {
 				if (is_array($regexp)) {
 					$stuff_to_parse = preg_replace(
-                        "#" .
-                        $regexp[GESHI_SEARCH] .
-                        "#{$regexp[GESHI_MODIFIERS]}",
+                        "/" .
+                        str_replace('/', '\/', $regexp[GESHI_SEARCH]) .
+                        "/{$regexp[GESHI_MODIFIERS]}",
                         "{$regexp[GESHI_BEFORE]}<|!REG3XP$key!>{$regexp[GESHI_REPLACE]}|>{$regexp[GESHI_AFTER]}",
                         $stuff_to_parse
                     );
 				} else {
-					$stuff_to_parse = preg_replace( "#(" . $regexp . ")#", "<|!REG3XP$key!>\\1|>", $stuff_to_parse);
+					$stuff_to_parse = preg_replace( "/(" . str_replace('/', '\/', $regexp) . ")/", "<|!REG3XP$key!>\\1|>", $stuff_to_parse);
 				}
 			}
 		}
@@ -1962,10 +2020,15 @@ class GeSHi
 		// almost exactly the same thing, except the second one prevents a number
 		// being highlighted twice (eg <span...><span...>5</span></span>)
 		// Put /NUM!/ in for the styles, which gets replaced at the end.
+        //
+        // NEW ONE: Brice Bernard
+        // $stuff_to_parse = preg_replace('/([^(\\w|#|\\\|"|\')])(\\d+)/', '\\1<|/NUM!/>\\2|>', $stuff_to_parse);
+        //$stuff_to_parse = preg_replace('/([-+]?\\b(?:[0-9]*\\.)?[0-9]+\\b)/', '<|/NUM!/>\\1|>', $stuff_to_parse);
 		//
 		if ($this->lexic_permissions['NUMBERS'] && preg_match('#[0-9]#', $stuff_to_parse )) {
-			$stuff_to_parse = preg_replace('#([^a-zA-Z0-9\#])([0-9]+)([^a-zA-Z0-9])#', "\\1<|/NUM!/>\\2|>\\3", $stuff_to_parse);
-			$stuff_to_parse = preg_replace('#([^a-zA-Z0-9\#>])([0-9]+)([^a-zA-Z0-9])#', "\\1<|/NUM!/>\\2|>\\3", $stuff_to_parse);
+			//$stuff_to_parse = preg_replace('#([^a-zA-Z0-9_\#])([0-9]+)([^a-zA-Z0-9])#', "\\1<|/NUM!/>\\2|>\\3", $stuff_to_parse);
+			//$stuff_to_parse = preg_replace('#([^a-zA-Z0-9_\#>])([0-9]+)([^a-zA-Z0-9])#', "\\1<|/NUM!/>\\2|>\\3", $stuff_to_parse);
+            $stuff_to_parse = preg_replace('/([-+]?\\b(?:[0-9]*\\.)?[0-9]+\\b)/', '<|/NUM!/>\\1|>', $stuff_to_parse);
 		}
 
 		// Highlight keywords
@@ -1974,31 +2037,32 @@ class GeSHi
 			foreach ($this->language_data['KEYWORDS'] as $k => $keywordset) {
 				if ($this->lexic_permissions['KEYWORDS'][$k]) {
 					foreach ($keywordset as $keyword) {
-						$keyword = quotemeta($keyword);
+						$keyword = preg_quote($keyword, '/');
 						//
 						// This replacement checks the word is on it's own (except if brackets etc
 						// are next to it), then highlights it. We don't put the color=" for the span
 						// in just yet - otherwise languages with the keywords "color" or "or" have
 						// a fit.
 						//
-						if (false !== stristr($stuff_to_parse, $keyword )) {
+						if (false !== stristr($stuff_to_parse_pregquote, $keyword )) {
 							$stuff_to_parse .= ' ';
 							// Might make a more unique string for putting the number in soon
 							// Basically, we don't put the styles in yet because then the styles themselves will
 							// get highlighted if the language has a CSS keyword in it (like CSS, for example ;))
 							$styles = "/$k/";
-							$keyword = quotemeta($keyword);
 							if ($this->language_data['CASE_SENSITIVE'][$k]) {
 								$stuff_to_parse = preg_replace(
-                                    "#([^a-zA-Z0-9\$_\|\#;>])($keyword)([^a-zA-Z0-9_<\|%\-&])#e",
-                                    "'\\1' . $func2('\\2', '$k', 'BEGIN') . '<|$styles>' . $func('\\2') . '|>' . $func2('\\2', '$k', 'END') . '\\3'",
+                                    "/([^a-zA-Z0-9\$_\|\#;>|^])($keyword)(?=[^a-zA-Z0-9_<\|%\-&])/e",
+                                    "'\\1' . $func2('\\2', '$k', 'BEGIN') . '<|$styles>' . $func('\\2') . '|>' . $func2('\\2', '$k', 'END')",
                                     $stuff_to_parse
                                 );
 							} else {
 								// Change the case of the word.
+                                // hackage again... must... release... 1.2...
+                                if ('smarty' == $this->language) { $hackage = '\/'; } else { $hackage = ''; }
 								$stuff_to_parse = preg_replace(
-                                    "#([^a-zA-Z0-9\$_\|\#;>])($keyword)([^a-zA-Z0-9_<\|%\-&])#ie",
-                                    "'\\1' . $func2('\\2', '$k', 'BEGIN') . '<|$styles>' . $func('\\2') . '|>' . $func2('\\2', '$k', 'END') . '\\3'",
+                                    "/([^a-zA-Z0-9\$_\|\#;>$hackage|^])($keyword)(?=[^a-zA-Z0-9_<\|%\-&])/ie",
+                                    "'\\1' . $func2('\\2', '$k', 'BEGIN') . '<|$styles>' . $func('\\2') . '|>' . $func2('\\2', '$k', 'END')",
                                     $stuff_to_parse
                                 );
 							}
@@ -2040,7 +2104,7 @@ class GeSHi
 					} else {
 						$attributes = ' class="me' . $key . '"';
 					}
-					$stuff_to_parse = preg_replace("#(" . quotemeta($this->language_data['OBJECT_SPLITTERS'][$key]) . "[\s]*)([a-zA-Z\*\(][a-zA-Z0-9_\*]*)#", "\\1<|$attributes>\\2|>", $stuff_to_parse);
+					$stuff_to_parse = preg_replace("/(" . preg_quote($this->language_data['OBJECT_SPLITTERS'][$key], 1) . "[\s]*)([a-zA-Z\*\(][a-zA-Z0-9_\*]*)/", "\\1<|$attributes>\\2|>", $stuff_to_parse);
 				}
 			}
 		}
@@ -2083,7 +2147,13 @@ class GeSHi
 				if (!$this->use_classes) {
 					$attributes = ' style="' . $this->language_data['STYLES']['REGEXPS'][$key] . '"';
 				} else {
-					$attributes = ' class="re' . $key . '"';
+                   if(is_array($this->language_data['REGEXPS'][$key]) &&
+                            array_key_exists(GESHI_CLASS, $this->language_data['REGEXPS'][$key])) {
+                        $attributes = ' class="'
+                            . $this->language_data['REGEXPS'][$key][GESHI_CLASS] . '"';
+                    } else {
+					   $attributes = ' class="re' . $key . '"';
+                    }
 				}
 				$stuff_to_parse = str_replace("!REG3XP$key!", "$attributes", $stuff_to_parse);
 			}
@@ -2109,7 +2179,7 @@ class GeSHi
 		$stuff_to_parse = str_replace('<|', '<span', $stuff_to_parse);
 		$stuff_to_parse = str_replace ( '|>', '</span>', $stuff_to_parse );
 
-		return substr(stripslashes($stuff_to_parse), 1);
+		return substr($stuff_to_parse, 1);
 	}
 
 	/**
@@ -2146,6 +2216,7 @@ class GeSHi
 	 */
 	function load_language ($file_name)
 	{
+        $this->enable_highlighting();
         $language_data = array();
 		require $file_name;
 		// Perhaps some checking might be added here later to check that
@@ -2166,7 +2237,6 @@ class GeSHi
         foreach ($this->language_data['REGEXPS'] as $key => $regexp) {
             $this->lexic_permissions['REGEXPS'][$key] = true;
         }
-		$this->enable_highlighting();
 		// Set default class for CSS
 		$this->overall_class = $this->language;
 	}
@@ -2195,6 +2265,16 @@ class GeSHi
             $parsed_code = $this->indent($parsed_code);
         }
         
+        // purge some unnecessary stuff
+        $parsed_code = preg_replace('#<span[^>]+>(\s*)</span>#', '\\1', $parsed_code);
+        $parsed_code = preg_replace('#<div[^>]+>(\s*)</div>#', '\\1', $parsed_code);
+
+        // If we are using IDs for line numbers, there needs to be an overall
+        // ID set to prevent collisions.
+        if ($this->add_ids && !$this->overall_id) {
+            $this->overall_id = 'geshi-' . substr(md5(microtime()), 0, 4);
+        }
+        
         // If we're using line numbers, we insert <li>s and appropriate
         // markup to style them (otherwise we don't need to do anything)
         if ($this->line_numbers != GESHI_NO_LINE_NUMBERS) {
@@ -2206,18 +2286,24 @@ class GeSHi
             // Set vars to defaults for following loop
             $parsed_code = '';
             $i = 0;
+            $attrs = array();
+                        
             // Foreach line...
-            foreach ($code as $line) {
-                $line = ( $line ) ? $line : '&nbsp;';
+            foreach ($code as $line) {//echo "LINE:|".htmlspecialchars($line)."| ".strlen($line)."<br />\n";
+                if ('' == $line || ' ' == $line) {
+                    $line = '&nbsp;';
+                }
                 // If this is a "special line"...
         	    if ($this->line_numbers == GESHI_FANCY_LINE_NUMBERS &&
                     $i % $this->line_nth_row == ($this->line_nth_row - 1)) {
             		// Set the attributes to style the line
                     if ($this->use_classes) {
-            			$attr = ' class="li2"';
+            			//$attr = ' class="li2"';
+                        $attrs['class'][] = 'li2';
             			$def_attr = ' class="de2"';
                     } else {
-            			$attr = ' style="' . $this->line_style2 . '"';
+            			//$attr = ' style="' . $this->line_style2 . '"';
+                        $attrs['style'][] = $this->line_style2;
             			// This style "covers up" the special styles set for special lines
             			// so that styles applied to special lines don't apply to the actual
             			// code on that line
@@ -2228,10 +2314,12 @@ class GeSHi
             		$end = '</div>';
             	} else {
             		if ($this->use_classes) {
-                        $attr = ' class="li1"';
+                        //$attr = ' class="li1"';
+                        $attrs['class'][] = 'li1';
             			$def_attr = ' class="de1"';
             		} else {
-                        $attr = ' style="' . $this->line_style1 . '"';
+                        //$attr = ' style="' . $this->line_style1 . '"';
+                        $attrs['style'][] = $this->line_style1;
             			$def_attr = ' style="' . $this->code_style . '"';
             		}
             		$start = "<div$def_attr>";
@@ -2241,17 +2329,23 @@ class GeSHi
             	++$i;
             	// Are we supposed to use ids? If so, add them
             	if ($this->add_ids) {
-            		$attr .= " id=\"{$this->overall_id}-{$i}\"";
+                    $attrs['id'][] = "$this->overall_id-$i";
             	}
             	if ($this->use_classes && in_array($i, $this->highlight_extra_lines)) {
-            		$attr .= " class=\"ln-xtra\"";
+                    $attrs['class'][] = 'ln-xtra';
             	}
             	if (!$this->use_classes && in_array($i, $this->highlight_extra_lines)) {
-            		$attr .= " style=\"{$this->highlight_extra_lines_style}\"";
+                    $attrs['style'][] = $this->highlight_extra_lines_style;
             	}
 
             	// Add in the line surrounded by appropriate list HTML
-            	$parsed_code .= "<li$attr>$start$line$end</li>$ls";
+                $attr_string = ' ';
+                foreach ($attrs as $key => $attr) {
+                    $attr_string .= $key . '="' . implode(' ', $attr) . '" ';
+                }
+                $attr_string = substr($attr_string, 0, -1);
+            	$parsed_code .= "<li$attr_string>$start$line$end</li>$ls";
+                $attrs = array();
         	}
         } else {
             // No line numbers, but still need to handle highlighting lines extra.
@@ -2259,27 +2353,25 @@ class GeSHi
             $code = explode("\n", $parsed_code);
             $parsed_code = '';
             $i = 0;
-            foreach ($code as $line)
-            {
+            foreach ($code as $line) {
             	// Make lines have at least one space in them if they're empty
-            	$line = ($line) ? $line : '&nbsp;';
+                if ('' == $line || ' ' == $line) {
+                    $line = '&nbsp;';
+                }
             	if (in_array(++$i, $this->highlight_extra_lines)) {
             		if ($this->use_classes) {
             			$parsed_code .= '<div class="ln-xtra">';
             		} else {
             			$parsed_code .= "<div style=\"{$this->highlight_extra_lines_style}\">";
             		}
-            		$parsed_code .= $line . "</div>\n";
+                    // Remove \n because it stuffs up <pre> header
+            		$parsed_code .= $line . "</div>";
             	} else {
             		$parsed_code .= $line . "\n";
             	}
         	}
         }
-        
-        // purge some unnecessary stuff
-        $parsed_code = preg_replace('#<span[^>]+>(\s*)</span>#', '\\1', $parsed_code);
-        $parsed_code = preg_replace('#<div[^>]+>(\s*)</div>#', '\\1', $parsed_code);
-        
+                
         if ($this->header_type == GESHI_HEADER_PRE) {
         	// enforce line numbers when using pre
             $parsed_code = str_replace('<li></li>', '<li>&nbsp;</li>', $parsed_code);
@@ -2405,7 +2497,7 @@ class GeSHi
 			if ($this->use_classes) {
 				$attr = ' class="foot"';
 			} else {
-				$attr = " style=\"{$this->footer_content_style}\">";
+				$attr = " style=\"{$this->footer_content_style}\"";
 			}
 			return "<div$attr>$footer</div>";
 		}
@@ -2598,7 +2690,15 @@ class GeSHi
 		foreach ($this->language_data['STYLES']['REGEXPS'] as $group => $styles) {
 			if (!$economy_mode || !($economy_mode && $styles == '') && !($economy_mode &&
                 !$this->lexic_permissions['REGEXPS'][$group])) {
-				$stylesheet .= "$selector.re$group {{$styles}}\n";
+                if (is_array($this->language_data['REGEXPS'][$group]) &&
+                         array_key_exists(GESHI_CLASS,
+                                    $this->language_data['REGEXPS'][$group])) {
+                    $stylesheet .= "$selector.";
+                    $stylesheet .= $this->language_data['REGEXPS'][$group][GESHI_CLASS];
+                    $stylesheet .= " {{$styles}}\n";
+                } else {
+				    $stylesheet .= "$selector.re$group {{$styles}}\n";
+                }
 			}
 		}
 
@@ -2620,7 +2720,7 @@ if (!function_exists('geshi_highlight')) {
      * @return string The code highlighted (if $return is true)
      * @since 1.0.2
      */
-	function geshi_highlight ($string, $language, $path, $return = false)
+	function geshi_highlight ($string, $language, $path = null, $return = false)
 	{
 		$geshi = new GeSHi($string, $language, $path);
 		$geshi->set_header_type(GESHI_HEADER_NONE);
@@ -2635,4 +2735,4 @@ if (!function_exists('geshi_highlight')) {
 	}
 }
 
-?>
+?>
\ No newline at end of file
