@@ -23,6 +23,8 @@
  */
 
 ini_set('max_execution_time', 18000);
+ini_set('log_errors', true);
+ini_set('display_errors', false);
 
 if(strpos(phpversion(), '4.4') === 0) {
 	error_reporting(E_ALL - E_NOTICE);
@@ -63,6 +65,7 @@ function usage($error = false) {
   -database <database>                   consider only queries on this database
   -user <user>                           consider only queries executed by this user
   -keepformatting                        keep the formatting of the query
+  -durationunit <s|ms>                   unit used to display the durations. Default is s(econds).
   -title <title>                         define the title of the reports
   -syslogident <ident>                   PostgreSQL syslog identity. Default is postgres.
   -memorylimit <n>                       PHP memory limit in MB. Default is 128.
@@ -207,6 +210,7 @@ $supportedReportBlocks = array(
 	'n-mostfrequent' => 'NormalizedQueriesMostFrequentReport',
 	'n-slowestaverage' => 'NormalizedQueriesSlowestAverageReport',
 	'history' => 'QueriesHistoryReport',
+	'historyperpid' => 'QueriesHistoryPerPidReport',
 	'n-mostfrequenterrors' => 'NormalizedErrorsMostFrequentReport',
 	'tsung' => 'TsungSessionsReport',
 );
@@ -319,15 +323,20 @@ if(isset($options['keepformatting'])) {
 	define('CONFIG_KEEP_FORMATTING', false);
 }
 
+if(isset($options['durationunit']) && $options['durationunit'] == 'ms') {
+	define('CONFIG_DURATION_UNIT', 'ms');
+} else {
+	define('CONFIG_DURATION_UNIT', 's');
+}
+
 if(isset($options['from']) && !empty($options['from'])) {
-	$fromTimestamp = strtotime($options['to']);
+	$fromTimestamp = strtotime($options['from']);
 	if($fromTimestamp <= 0) {
 		$fromTimestamp = false;
 	}
 } else {
 	$fromTimestamp = false;
 }
-define('CONFIG_FROM_TIMESTAMP', $fromTimestamp);
 
 if(isset($options['to']) && !empty($options['to'])) {
 	$toTimestamp = strtotime($options['to']);
@@ -337,10 +346,19 @@ if(isset($options['to']) && !empty($options['to'])) {
 } else {
 	$toTimestamp = false;
 }
-define('CONFIG_TO_TIMESTAMP', $toTimestamp);
 
-if(CONFIG_FROM_TIMESTAMP || CONFIG_TO_TIMESTAMP) {
+if($fromTimestamp || $toTimestamp) {
 	define('CONFIG_TIMESTAMP_FILTER', true);
+	if($fromTimestamp) {
+		define('CONFIG_FROM_TIMESTAMP', $fromTimestamp);
+	} else {
+		define('CONFIG_FROM_TIMESTAMP', MIN_TIMESTAMP);
+	}
+	if($toTimestamp) {
+		define('CONFIG_TO_TIMESTAMP', $toTimestamp);
+	} else {
+		define('CONFIG_TO_TIMESTAMP', MAX_TIMESTAMP);
+	}
 } else {
 	define('CONFIG_TIMESTAMP_FILTER', false);
 }
@@ -367,4 +385,6 @@ $logReader->output();
 
 fclose($stderr);
 
+exit(0);
+
 ?>
\ No newline at end of file
